"""
Notion Markdown Exporter

NotionページをMarkdown形式にエクスポートするツール。
すべての主要なブロックタイプをサポートし、親子関係を適切に反映します。

サポートされるブロックタイプ:
- テキストブロック: 段落、見出し（H1/H2/H3）、引用
- リスト: 箇条書き、番号付き、ToDoリスト（ネスト対応）
- コード: コードブロック、インラインコード
- 数式: ブロック数式、インライン数式
- テーブル: ヘッダー付きテーブル
- メディア: 画像、動画、音声、PDF、ファイル、ブックマーク、埋め込み
- レイアウト: カラム、区切り線、目次
- 特殊: トグル、コールアウト、同期ブロック、子ページ、子データベース

使用方法:
    python export_as_markdown.py --page-id YOUR_PAGE_ID --output output.md

必要な環境変数:
    NOTION_API_TOKEN: NotionインテグレーションのAPIトークン
"""

import argparse
import asyncio
import os

from dotenv import load_dotenv

from notion_py_client import NotionAsyncClient
from notion_py_client.models.rich_text_item import rich_text_to_markdown
from notion_py_client.utils import is_full_block

load_dotenv()  # Load environment variables from .env file

client = NotionAsyncClient(auth=os.environ["NOTION_API_TOKEN"])


async def convert_block_to_markdown(
    block_id: str, indent_level: int = 0, debug: bool = False
) -> str:
    """
    Notionブロックをマークダウンに変換する

    Args:
        block_id: 変換するブロックのID
        indent_level: インデントレベル（ネストされた要素用）
        debug: デバッグモード

    Returns:
        マークダウン形式の文字列
    """
    block = await client.blocks.retrieve(block_id=block_id)

    if not is_full_block(block):
        return ""

    indent = "  " * indent_level  # 2スペースでインデント
    children_markdown = ""

    # 子要素を持つ場合は再帰的に処理
    if block.has_children:
        children = await client.blocks.children.list(block_id=block_id)

        for i, child in enumerate(children.results):
            if is_full_block(child):
                # 子要素のインデントレベルを決定
                child_indent = indent_level + 1
                child_md = await convert_block_to_markdown(
                    child.id, child_indent, debug
                )
                children_markdown += child_md

    # ブロックタイプごとにマークダウンを生成
    result = ""

    # テキストブロック
    if block.type == "paragraph":
        text = rich_text_to_markdown(block.paragraph.rich_text)
        if text or children_markdown:
            result = f"{indent}{text}\n"
            if children_markdown:
                result += children_markdown
            result += "\n" if not children_markdown else ""

    elif block.type == "heading_1":
        text = rich_text_to_markdown(block.heading_1.rich_text)
        # トグル可能な見出しの場合
        if block.heading_1.is_toggleable and children_markdown:
            result = f"{indent}<details>\n{indent}<summary># {text}</summary>\n\n"
            result += children_markdown
            result += f"{indent}</details>\n\n"
        else:
            result = f"{indent}# {text}\n\n"
            if children_markdown:
                result += children_markdown

    elif block.type == "heading_2":
        text = rich_text_to_markdown(block.heading_2.rich_text)
        # トグル可能な見出しの場合
        if block.heading_2.is_toggleable and children_markdown:
            result = f"{indent}<details>\n{indent}<summary>## {text}</summary>\n\n"
            result += children_markdown
            result += f"{indent}</details>\n\n"
        else:
            result = f"{indent}## {text}\n\n"
            if children_markdown:
                result += children_markdown

    elif block.type == "heading_3":
        text = rich_text_to_markdown(block.heading_3.rich_text)
        # トグル可能な見出しの場合
        if block.heading_3.is_toggleable and children_markdown:
            result = f"{indent}<details>\n{indent}<summary>### {text}</summary>\n\n"
            result += children_markdown
            result += f"{indent}</details>\n\n"
        else:
            result = f"{indent}### {text}\n\n"
            if children_markdown:
                result += children_markdown

    # リストブロック
    elif block.type == "bulleted_list_item":
        text = rich_text_to_markdown(block.bulleted_list_item.rich_text)
        result = f"{indent}- {text}\n"
        if children_markdown:
            result += children_markdown

    elif block.type == "numbered_list_item":
        text = rich_text_to_markdown(block.numbered_list_item.rich_text)
        result = f"{indent}1. {text}\n"
        if children_markdown:
            result += children_markdown

    # ToDoブロック
    elif block.type == "to_do":
        checked = "x" if block.to_do.checked else " "
        text = rich_text_to_markdown(block.to_do.rich_text)
        result = f"{indent}- [{checked}] {text}\n"
        if children_markdown:
            result += children_markdown

    # 引用ブロック
    elif block.type == "quote":
        text = rich_text_to_markdown(block.quote.rich_text)
        result = f"{indent}> {text}\n"
        if children_markdown:
            # 引用の中の子要素も引用として扱う
            for line in children_markdown.split("\n"):
                if line:
                    result += f"{indent}> {line}\n"
        result += "\n"

    # トグルブロック
    elif block.type == "toggle":
        text = rich_text_to_markdown(block.toggle.rich_text)
        result = f"{indent}<details>\n{indent}<summary>{text}</summary>\n\n"
        if children_markdown:
            result += children_markdown
        result += f"{indent}</details>\n\n"

    # コールアウトブロック
    elif block.type == "callout":
        text = rich_text_to_markdown(block.callout.rich_text)
        icon = ""
        if block.callout.icon:
            if block.callout.icon.type == "emoji":
                icon = block.callout.icon.emoji + " "
        result = f"{indent}> {icon}{text}\n"
        if children_markdown:
            for line in children_markdown.split("\n"):
                if line:
                    result += f"{indent}> {line}\n"
        result += "\n"

    # コードブロック
    elif block.type == "code":
        text = rich_text_to_markdown(block.code.rich_text)
        lang = block.code.language
        result = f"{indent}```{lang}\n{text}\n{indent}```\n\n"
        if children_markdown:
            result += children_markdown

    # 数式ブロック
    elif block.type == "equation":
        result = f"{indent}$$\n{block.equation.expression}\n{indent}$$\n\n"
        if children_markdown:
            result += children_markdown

    # 区切り線
    elif block.type == "divider":
        result = f"{indent}---\n\n"

    # レイアウトブロック
    elif block.type == "column_list":
        # カラムリストは子要素（カラム）をそのまま表示
        if children_markdown:
            result = children_markdown

    elif block.type == "column":
        # カラムの内容をそのまま表示
        if children_markdown:
            result = children_markdown

    # テーブル
    elif block.type == "table":
        # テーブルは子要素（table_row）で構成される
        if children_markdown:
            lines = children_markdown.strip().split("\n")
            if len(lines) > 0:
                # 最初の行をヘッダーとして扱う
                header = lines[0].strip()
                # カラム数を数える
                col_count = header.count("|") - 1
                separator = indent + "|" + " --- |" * col_count

                # ヘッダー、区切り線、その他の行を結合
                result = indent + header + "\n" + separator + "\n"
                if len(lines) > 1:
                    for line in lines[1:]:
                        if line.strip():
                            result += indent + line.strip() + "\n"
                result += "\n"

    elif block.type == "table_row":
        # テーブル行（インデントなしで出力）
        cells = [rich_text_to_markdown(cell) for cell in block.table_row.cells]
        result = f"| {' | '.join(cells)} |\n"

    # メディアブロック
    elif block.type == "image":
        caption = (
            rich_text_to_markdown(block.image.caption) if block.image.caption else ""
        )
        url = (
            block.image.external.url
            if block.image.type == "external"
            else block.image.file.url
        )
        alt_text = caption if caption else "image"
        result = f"{indent}![{alt_text}]({url})\n\n"
        if children_markdown:
            result += children_markdown

    elif block.type == "video":
        caption = (
            rich_text_to_markdown(block.video.caption) if block.video.caption else ""
        )
        url = (
            block.video.external.url
            if block.video.type == "external"
            else block.video.file.url
        )
        result = f"{indent}[Video: {caption if caption else 'video'}]({url})\n\n"
        if children_markdown:
            result += children_markdown

    elif block.type == "file":
        caption = block.file.name if block.file.name else ""
        url = (
            block.file.external.url
            if block.file.type == "external"
            else block.file.file.url
        )
        result = f"{indent}[File: {caption if caption else 'file'}]({url})\n\n"
        if children_markdown:
            result += children_markdown

    elif block.type == "pdf":
        caption = rich_text_to_markdown(block.pdf.caption) if block.pdf.caption else ""
        url = (
            block.pdf.external.url
            if block.pdf.type == "external"
            else block.pdf.file.url
        )
        result = f"{indent}[PDF: {caption if caption else 'pdf'}]({url})\n\n"
        if children_markdown:
            result += children_markdown

    elif block.type == "audio":
        caption = (
            rich_text_to_markdown(block.audio.caption) if block.audio.caption else ""
        )
        url = (
            block.audio.external.url
            if block.audio.type == "external"
            else block.audio.file.url
        )
        result = f"{indent}[Audio: {caption if caption else 'audio'}]({url})\n\n"
        if children_markdown:
            result += children_markdown

    elif block.type == "bookmark":
        caption = (
            rich_text_to_markdown(block.bookmark.caption)
            if block.bookmark.caption
            else ""
        )
        url = block.bookmark.url
        result = f"{indent}[Bookmark: {caption if caption else url}]({url})\n\n"
        if children_markdown:
            result += children_markdown

    elif block.type == "embed":
        caption = (
            rich_text_to_markdown(block.embed.caption) if block.embed.caption else ""
        )
        url = block.embed.url
        result = f"{indent}[Embed: {caption if caption else url}]({url})\n\n"
        if children_markdown:
            result += children_markdown

    elif block.type == "link_preview":
        url = block.link_preview.url
        result = f"{indent}[Link Preview]({url})\n\n"

    # 特殊ブロック
    elif block.type == "child_page":
        result = f"{indent}[Page: {block.child_page.title}]\n\n"
        if children_markdown:
            result += children_markdown

    elif block.type == "child_database":
        result = f"{indent}[Database: {block.child_database.title}]\n\n"
        if children_markdown:
            result += children_markdown

    elif block.type == "table_of_contents":
        result = f"{indent}[Table of Contents]\n\n"

    elif block.type == "breadcrumb":
        result = f"{indent}[Breadcrumb]\n\n"

    elif block.type == "template":
        text = rich_text_to_markdown(block.template.rich_text)
        result = f"{indent}[Template: {text}]\n\n"
        if children_markdown:
            result += children_markdown

    elif block.type == "synced_block":
        # 同期ブロックの処理
        if block.synced_block.synced_from:
            synced_block_id = block.synced_block.synced_from.block_id
            synced_content = await convert_block_to_markdown(
                synced_block_id, indent_level, debug
            )
            result = synced_content
        else:
            # オリジナルの同期ブロック
            if children_markdown:
                result = children_markdown

    elif block.type == "link_to_page":
        page_info = block.link_to_page
        if page_info.page_id:
            result = f"{indent}[Link to Page](https://notion.so/{page_info.page_id.replace('-', '')})\n\n"
        elif page_info.database_id:
            result = f"{indent}[Link to Database](https://notion.so/{page_info.database_id.replace('-', '')})\n\n"

    # 未サポートのブロック
    elif block.type == "unsupported":
        result = f"{indent}[Unsupported Block]\n\n"

    else:
        # その他の未知のブロックタイプ
        result = f"{indent}[Unknown Block Type: {block.type}]\n\n"
        if children_markdown:
            result += children_markdown

    return result


async def export_page_as_markdown(
    page_id: str, output_file: str = None, debug: bool = False
) -> str:
    """
    Notionページ全体をマークダウンにエクスポートする

    Args:
        page_id: エクスポートするページのID
        output_file: 出力ファイルパス（オプション）
        debug: デバッグモード

    Returns:
        マークダウン形式の文字列
    """
    # ページ情報を取得
    page = await client.pages.retrieve({"page_id": page_id})

    # ページタイトルを取得
    for property in page.properties.values():
        if property.type == "title":
            title = rich_text_to_markdown(property.title)
            break
    else:
        title = ""
    markdown = f"# {title}\n\n"

    # ページの子ブロックを取得
    children = await client.blocks.children.list(block_id=page_id)

    for child in children.results:
        if is_full_block(child):
            child_markdown = await convert_block_to_markdown(
                child.id, indent_level=0, debug=debug
            )
            markdown += child_markdown

    # ファイルに出力
    if output_file:
        with open(output_file, "w", encoding="utf-8") as f:
            f.write(markdown)
        print(f"マークダウンファイルを出力しました: {output_file}")

    return markdown


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--page-id", type=str, required=True)
    parser.add_argument("--output", type=str, required=False, default="output.md")
    parser.add_argument("--debug", action="store_true", default=False)
    args = parser.parse_args()
    asyncio.run(export_page_as_markdown(args.page_id, args.output, args.debug))



"""
特殊ブロックの定義

SyncedBlock, ChildPage, ChildDatabase, Equation, Code, Callout
"""

from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field, StrictStr

from .base import ApiColor, BaseBlockObject
from ..models.rich_text_item import RichTextItem
from ..models.icon import NotionIcon


# CodeLanguage as Literal (TypeScript-style)
CodeLanguage = Literal[
    "abap",
    "abc",
    "agda",
    "arduino",
    "ascii art",
    "assembly",
    "bash",
    "basic",
    "bnf",
    "c",
    "c#",
    "c++",
    "clojure",
    "coffeescript",
    "coq",
    "css",
    "dart",
    "dhall",
    "diff",
    "docker",
    "ebnf",
    "elixir",
    "elm",
    "erlang",
    "f#",
    "flow",
    "fortran",
    "gherkin",
    "glsl",
    "go",
    "graphql",
    "groovy",
    "haskell",
    "hcl",
    "html",
    "idris",
    "java",
    "javascript",
    "json",
    "julia",
    "kotlin",
    "latex",
    "less",
    "lisp",
    "livescript",
    "llvm ir",
    "lua",
    "makefile",
    "markdown",
    "markup",
    "matlab",
    "mathematica",
    "mermaid",
    "nix",
    "notion formula",
    "objective-c",
    "ocaml",
    "pascal",
    "perl",
    "php",
    "plain text",
    "powershell",
    "prolog",
    "protobuf",
    "purescript",
    "python",
    "r",
    "racket",
    "reason",
    "ruby",
    "rust",
    "sass",
    "scala",
    "scheme",
    "scss",
    "shell",
    "smalltalk",
    "solidity",
    "sql",
    "swift",
    "toml",
    "typescript",
    "vb.net",
    "verilog",
    "vhdl",
    "visual basic",
    "webassembly",
    "xml",
    "yaml",
    "java/c/c++/c#",
]


# ============================================
# Content Models
# ============================================


class SyncedBlockContent(BaseModel):
    """同期ブロックコンテンツ"""

    synced_from: SyncedFromBlock | None = Field(None, description="同期元ブロック")


class SyncedFromBlock(BaseModel):
    """同期元ブロック情報"""

    type: Literal["block_id"] = Field("block_id", description="タイプ")
    block_id: StrictStr = Field(..., description="ブロックID")


class TitleObject(BaseModel):
    """タイトルオブジェクト"""

    title: StrictStr = Field(..., description="タイトル")


class ExpressionObject(BaseModel):
    """数式オブジェクト"""

    expression: StrictStr = Field(..., description="数式")


class CodeContent(BaseModel):
    """コードコンテンツ"""

    rich_text: list[RichTextItem] = Field(..., description="リッチテキスト配列")
    caption: list[RichTextItem] = Field(..., description="キャプション")
    language: CodeLanguage = Field(..., description="プログラミング言語")


class CalloutContent(BaseModel):
    """コールアウトコンテンツ"""

    rich_text: list[RichTextItem] = Field(..., description="リッチテキスト配列")
    color: ApiColor = Field(..., description="カラー設定")
    icon: NotionIcon | None = Field(None, description="アイコン")


# ============================================
# Special Blocks
# ============================================


class SyncedBlockBlock(BaseBlockObject):
    """同期ブロック"""

    type: Literal["synced_block"] = Field("synced_block", description="ブロックタイプ")
    synced_block: SyncedBlockContent = Field(..., description="同期ブロックコンテンツ")

class ChildPageBlock(BaseBlockObject):
    """子ページブロック"""

    type: Literal["child_page"] = Field("child_page", description="ブロックタイプ")
    child_page: TitleObject = Field(..., description="子ページ情報")

class ChildDatabaseBlock(BaseBlockObject):
    """子データベースブロック"""

    type: Literal["child_database"] = Field(
        "child_database", description="ブロックタイプ"
    )
    child_database: TitleObject = Field(..., description="子データベース情報")

class EquationBlock(BaseBlockObject):
    """数式ブロック"""

    type: Literal["equation"] = Field("equation", description="ブロックタイプ")
    equation: ExpressionObject = Field(..., description="数式")

class CodeBlock(BaseBlockObject):
    """コードブロック"""

    type: Literal["code"] = Field("code", description="ブロックタイプ")
    code: CodeContent = Field(..., description="コードコンテンツ")

class CalloutBlock(BaseBlockObject):
    """コールアウトブロック"""

    type: Literal["callout"] = Field("callout", description="ブロックタイプ")
    callout: CalloutContent = Field(..., description="コールアウトコンテンツ")




from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class CheckboxPropertyConfig(
    BasePropertyConfig[Literal["checkbox"]]
):
    """Notionのcheckboxプロパティ設定"""

    type: Literal["checkbox"] = Field(
        "checkbox", description="プロパティタイプ"
    )
    checkbox: EmptyObject = Field(
        default_factory=EmptyObject, description="checkbox設定"
    )



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class TitlePropertyConfig(BasePropertyConfig[Literal["title"]]):
    """Notionのtitleプロパティ設定"""

    type: Literal["title"] = Field(
        "title", description="プロパティタイプ"
    )
    title: EmptyObject = Field(
        default_factory=EmptyObject, description="title設定"
    )



from typing import Literal

from pydantic import BaseModel, Field, StrictStr

from ._base_config import BasePropertyConfig


class UniqueIdOptions(BaseModel):
    """unique_idプロパティの設定"""

    prefix: StrictStr | None = Field(None, description="ユニークIDのプレフィックス")


class UniqueIdPropertyConfig(
    BasePropertyConfig[Literal["unique_id"]]
):
    """Notionのunique_idプロパティ設定"""

    type: Literal["unique_id"] = Field(
        "unique_id", description="プロパティタイプ"
    )
    unique_id: UniqueIdOptions = Field(
        default_factory=lambda: UniqueIdOptions(prefix=None),
        description="unique_id設定",
    )



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class ButtonPropertyConfig(BasePropertyConfig[Literal["button"]]):
    """Notionのbuttonプロパティ設定"""

    type: Literal["button"] = Field(
        "button", description="プロパティタイプ"
    )
    button: EmptyObject = Field(default_factory=EmptyObject, description="button設定")



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class VerificationPropertyConfig(
    BasePropertyConfig[Literal["verification"]]
):
    """Notionのverificationプロパティ設定"""

    type: Literal["verification"] = Field(
        "verification", description="プロパティタイプ"
    )
    verification: EmptyObject = Field(
        default_factory=EmptyObject, description="verification設定"
    )



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class PeoplePropertyConfig(BasePropertyConfig[Literal["people"]]):
    """Notionのpeopleプロパティ設定"""

    type: Literal["people"] = Field(
        "people", description="プロパティタイプ"
    )
    people: EmptyObject = Field(
        default_factory=EmptyObject, description="people設定"
    )



"""
Notionブロックの基底クラスと共通モデル
"""

from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field, StrictBool, StrictStr

from ..models.parent import NotionParent
from ..models.user import PartialUser

# BlockType as Literal (TypeScript-style)
BlockType = Literal[
    "paragraph",
    "heading_1",
    "heading_2",
    "heading_3",
    "bulleted_list_item",
    "numbered_list_item",
    "quote",
    "to_do",
    "toggle",
    "template",
    "synced_block",
    "child_page",
    "child_database",
    "equation",
    "code",
    "callout",
    "divider",
    "breadcrumb",
    "table_of_contents",
    "column_list",
    "column",
    "link_to_page",
    "table",
    "table_row",
    "embed",
    "bookmark",
    "image",
    "video",
    "pdf",
    "file",
    "audio",
    "link_preview",
    "unsupported",
]

# ApiColor as Literal (TypeScript-style)
ApiColor = Literal[
    "default",
    "gray",
    "brown",
    "orange",
    "yellow",
    "green",
    "blue",
    "purple",
    "pink",
    "red",
    "default_background",
    "gray_background",
    "brown_background",
    "orange_background",
    "yellow_background",
    "green_background",
    "blue_background",
    "purple_background",
    "pink_background",
    "red_background",
]


class BaseBlockObject(BaseModel):
    """
    すべてのブロックタイプに共通のフィールド

    各ブロックタイプは、このクラスを継承し、
    `type`フィールドをLiteralで定義します。
    """

    object: Literal["block"] = Field("block", description="オブジェクトタイプ")
    id: StrictStr = Field(..., description="ブロックID")
    # type field is defined in each subclass with specific Literal
    created_time: StrictStr = Field(..., description="作成日時（ISO 8601形式）")
    created_by: PartialUser = Field(..., description="作成者")
    last_edited_time: StrictStr = Field(..., description="最終編集日時（ISO 8601形式）")
    last_edited_by: PartialUser = Field(..., description="最終編集者")
    parent: NotionParent = Field(..., description="親オブジェクト")
    has_children: StrictBool = Field(False, description="子ブロックの有無")
    archived: StrictBool = Field(False, description="アーカイブフラグ")
    in_trash: StrictBool = Field(False, description="ゴミ箱フラグ")


class PartialBlock(BaseModel):
    """部分的なブロック情報"""

    object: Literal["block"] = Field("block", description="オブジェクトタイプ")
    id: StrictStr = Field(..., description="ブロックID")



"""
サポートされていないブロックの定義
"""

from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field

from .base import BaseBlockObject


class EmptyObject(BaseModel):
    """空のオブジェクト"""

    pass


class UnsupportedBlock(BaseBlockObject):
    """サポートされていないブロック"""

    type: Literal["unsupported"] = Field("unsupported", description="ブロックタイプ")
    unsupported: EmptyObject = Field(
        default_factory=EmptyObject, description="空のコンテンツ"
    )

    def to_markdown(self) -> str:
        """サポートされていないブロックをMarkdown形式に変換"""
        return ""



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class EmailPropertyConfig(BasePropertyConfig[Literal["email"]]):
    """Notionのemailプロパティ設定"""

    type: Literal["email"] = Field(
        "email", description="プロパティタイプ"
    )
    email: EmptyObject = Field(
        default_factory=EmptyObject, description="email設定"
    )



"""
Notionリストレスポンス型の定義

TypeScript定義: ListUsersResponse, QueryDatabaseResponse, ListBlockChildrenResponse等
"""

from __future__ import annotations

from typing import Generic, Literal, TypeVar

from pydantic import BaseModel, Field, StrictBool, StrictStr

from .page import NotionPage, PartialPage
from .database import NotionDatabase, PartialDatabase
from .datasource import DataSource, PartialDataSource
from .file_upload import FileUploadObject
from ..models.user import PartialUser
from ..models.parent import NotionParent
from ..models.rich_text_item import RichTextItem
from ..blocks.base import PartialBlock
from ..blocks import BlockObject

T = TypeVar("T")


# ============ Comment Type ============
class CommentObject(BaseModel):
    """Notionのコメントオブジェクト

    TypeScript: CommentObjectResponse
    """

    object: Literal["comment"] = Field("comment", description="オブジェクトタイプ")
    id: StrictStr = Field(..., description="コメントID")
    parent: NotionParent = Field(
        ..., description="親オブジェクト（ページまたはブロック）"
    )
    discussion_id: StrictStr = Field(..., description="ディスカッションID")
    created_time: StrictStr = Field(..., description="作成日時（ISO 8601形式）")
    last_edited_time: StrictStr = Field(..., description="最終編集日時（ISO 8601形式）")
    created_by: PartialUser = Field(..., description="作成者")
    rich_text: list[RichTextItem] = Field(..., description="コメントテキスト")


# ============ List Response Types ============


class ListResponse(BaseModel, Generic[T]):
    """汎用リストレスポンス.

    Notionの全てのリストAPIで共通のページネーション構造。

    Attributes:
        object: 常に "list"
        results: 結果の配列
        next_cursor: 次のページのカーソル（なければNone）
        has_more: さらにページがあるかどうか

    Examples:
        >>> response = await client.databases.query({"database_id": "abc"})
        >>> print(f"Has more: {response.has_more}")
        >>> for page in response.results:
        ...     print(page.id)
    """

    object: Literal["list"] = Field(..., description="オブジェクトタイプ")
    results: list[T] = Field(..., description="結果の配列")
    next_cursor: StrictStr | None = Field(None, description="次のページのカーソル")
    has_more: StrictBool = Field(..., description="さらにページがあるか")
    type: StrictStr | None = Field(
        None, description="結果の型ヒント（例: 'page_or_database'）"
    )


# 具体的なリストレスポンス型
class QueryDatabaseResponse(ListResponse[NotionPage]):
    """databases.query() のレスポンス型.

    備考: リスト型の共通フィールド `type` は親クラスで定義済み。
    サブクラスではオーバーライドしない（LSP/型チェッカー警告を避ける）。
    """


class QueryDataSourceResponse(
    ListResponse[NotionPage | PartialPage | DataSource | PartialDataSource]
):
    """dataSources.query() のレスポンス型.

    備考: `type` は親クラス側の型（StrictStr | None）を使用。
    """


class ListUsersResponse(ListResponse[PartialUser]):
    """users.list() のレスポンス型.

    備考: `type` は親クラスに準拠。
    """


class ListBlockChildrenResponse(ListResponse[BlockObject | PartialBlock]):
    """blocks.children.list() のレスポンス型.

    備考: ブロックは完全なオブジェクト(BaseBlockObject)または部分的なオブジェクト(PartialBlock)が返される。
    """


class ListCommentsResponse(ListResponse[CommentObject]):
    """comments.list() のレスポンス型.

    備考: コメントのリストを返す。
    """


class SearchResponse(
    ListResponse[NotionPage | PartialPage | NotionDatabase | PartialDatabase]
):
    """search() のレスポンス型.

    備考: `type` は親クラスに準拠。
    """


class ListFileUploadsResponse(ListResponse[FileUploadObject]):
    """file_uploads.list() のレスポンス型.

    備考: `type` は親クラスに準拠（省略される場合あり）。
    """



"""
メディア系ブロックの定義

Embed, Bookmark, Image, Video, PDF, File, Audio, LinkPreview
"""

from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field, StrictStr

from .base import BaseBlockObject
from ..models.rich_text_item import RichTextItem
from ..models.file import InternalFile


# MediaFileType as Literal (TypeScript-style)
MediaFileType = Literal["external", "file"]


# ============================================
# Content Models
# ============================================


class MediaContentWithUrlAndCaption(BaseModel):
    """URLとキャプションを持つメディアコンテンツ"""

    url: StrictStr = Field(..., description="URL")
    caption: list[RichTextItem] = Field(..., description="キャプション")


class ExternalFileReference(BaseModel):
    """外部ファイル参照"""

    url: StrictStr = Field(..., description="URL")


class ExternalMediaContentWithFileAndCaption(BaseModel):
    """外部ファイルとキャプションを持つメディアコンテンツ"""

    type: Literal["external"] = Field("external", description="ファイルタイプ")
    external: ExternalFileReference = Field(..., description="外部ファイル")
    caption: list[RichTextItem] = Field(..., description="キャプション")


class FileMediaContentWithFileAndCaption(BaseModel):
    """内部ファイルとキャプションを持つメディアコンテンツ"""

    type: Literal["file"] = Field("file", description="ファイルタイプ")
    file: InternalFile = Field(..., description="内部ファイル")
    caption: list[RichTextItem] = Field(..., description="キャプション")


MediaContentWithFileAndCaption = (
    ExternalMediaContentWithFileAndCaption | FileMediaContentWithFileAndCaption
)


class ExternalMediaContentWithFileNameAndCaption(BaseModel):
    """外部ファイル（ファイル名付き）とキャプションを持つメディアコンテンツ"""

    type: Literal["external"] = Field("external", description="ファイルタイプ")
    external: ExternalFileReference = Field(..., description="外部ファイル")
    caption: list[RichTextItem] = Field(..., description="キャプション")
    name: StrictStr = Field(..., description="ファイル名")


class FileMediaContentWithFileNameAndCaption(BaseModel):
    """内部ファイル（ファイル名付き）とキャプションを持つメディアコンテンツ"""

    type: Literal["file"] = Field("file", description="ファイルタイプ")
    file: InternalFile = Field(..., description="内部ファイル")
    caption: list[RichTextItem] = Field(..., description="キャプション")
    name: StrictStr = Field(..., description="ファイル名")


MediaContentWithFileNameAndCaption = (
    ExternalMediaContentWithFileNameAndCaption | FileMediaContentWithFileNameAndCaption
)


class MediaContentWithUrl(BaseModel):
    """URLのみを持つメディアコンテンツ"""

    url: StrictStr = Field(..., description="URL")


# ============================================
# Media Blocks
# ============================================


class EmbedBlock(BaseBlockObject):
    """埋め込みブロック"""

    type: Literal["embed"] = Field("embed", description="ブロックタイプ")
    embed: MediaContentWithUrlAndCaption = Field(..., description="埋め込みコンテンツ")


class BookmarkBlock(BaseBlockObject):
    """ブックマークブロック"""

    type: Literal["bookmark"] = Field("bookmark", description="ブロックタイプ")
    bookmark: MediaContentWithUrlAndCaption = Field(
        ..., description="ブックマークコンテンツ"
    )

class ImageBlock(BaseBlockObject):
    """画像ブロック"""

    type: Literal["image"] = Field("image", description="ブロックタイプ")
    image: MediaContentWithFileAndCaption = Field(..., description="画像コンテンツ")

class VideoBlock(BaseBlockObject):
    """動画ブロック"""

    type: Literal["video"] = Field("video", description="ブロックタイプ")
    video: MediaContentWithFileAndCaption = Field(..., description="動画コンテンツ")


class PdfBlock(BaseBlockObject):
    """PDFブロック"""

    type: Literal["pdf"] = Field("pdf", description="ブロックタイプ")
    pdf: MediaContentWithFileAndCaption = Field(..., description="PDFコンテンツ")

class FileBlock(BaseBlockObject):
    """ファイルブロック"""

    type: Literal["file"] = Field("file", description="ブロックタイプ")
    file: MediaContentWithFileNameAndCaption = Field(
        ..., description="ファイルコンテンツ"
    )


class AudioBlock(BaseBlockObject):
    """音声ブロック"""

    type: Literal["audio"] = Field("audio", description="ブロックタイプ")
    audio: MediaContentWithFileAndCaption = Field(..., description="音声コンテンツ")


class LinkPreviewBlock(BaseBlockObject):
    """リンクプレビューブロック"""

    type: Literal["link_preview"] = Field("link_preview", description="ブロックタイプ")
    link_preview: MediaContentWithUrl = Field(
        ..., description="リンクプレビューコンテンツ"
    )



from typing import Any, Literal
import json

from pydantic import Field

from ._base_property import BaseProperty


class PlaceProperty(BaseProperty[Literal["place"]]):
    """Notionのplaceプロパティ

    返却形状は仕様追加中のため、値は汎用の辞書として保持します。
    """

    type: Literal["place"] = Field("place", description="プロパティタイプ")

    place: dict[str, Any] | None = Field(
        default=None, description="place情報（API仕様未確定のため汎用辞書）"
    )

    def get_display_value(self) -> str | None:
        """place情報を取得

        Returns:
            str | None: place情報の文字列表現。未設定の場合はNone
        """
        if self.place is None:
            return None
        try:
            return json.dumps(self.place, ensure_ascii=False, separators=(",", ":"))
        except Exception:
            return str(self.place)



from abc import abstractmethod
from typing import Generic, TypeVar, Literal

from pydantic import BaseModel, Field, StrictStr


# Notion プロパティタイプの文字列リテラル型定義
NotionPropertyType = Literal[
    "title",
    "rich_text",
    "date",
    "number",
    "select",
    "multi_select",
    "people",
    "relation",
    "url",
    "checkbox",
    "formula",
    "status",
    "rollup",
    "button",
    "last_edited_time",
    "email",
    "phone_number",
    "files",
    "created_by",
    "created_time",
    "last_edited_by",
    "unique_id",
    "verification",
    "location",
    "last_visited_time",
    "place",
]

TPropertyType = TypeVar("TPropertyType", bound=NotionPropertyType)


class BaseProperty(BaseModel, Generic[TPropertyType]):
    """Notionプロパティのベースクラス"""

    id: StrictStr | None = Field(None, description="プロパティID")
    # 各サブクラスで特定の Literal[...] によって具体化される
    type: TPropertyType = Field(..., description="プロパティタイプ")

    @abstractmethod
    def get_display_value(self) -> str | int | float | bool | None:
        """Notion UI上でのプロパティの表示用の値を取得"""
        pass



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class LocationPropertyConfig(
    BasePropertyConfig[Literal["location"]]
):
    """Notionのlocationプロパティ設定

    TypeScript: LocationPropertyConfigurationRequest
    フィールドは空オブジェクトで構成されます。
    """

    type: Literal["location"] = Field(
        "location", description="プロパティタイプ"
    )
    location: EmptyObject = Field(
        default_factory=EmptyObject, description="location設定"
    )




from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class PhoneNumberPropertyConfig(
    BasePropertyConfig[Literal["phone_number"]]
):
    """Notionのphone_numberプロパティ設定"""

    type: Literal["phone_number"] = Field(
        "phone_number", description="プロパティタイプ"
    )
    phone_number: EmptyObject = Field(
        default_factory=EmptyObject, description="phone_number設定"
    )



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class UrlPropertyConfig(BasePropertyConfig[Literal["url"]]):
    """Notionのurlプロパティ設定"""

    type: Literal["url"] = Field(
        "url", description="プロパティタイプ"
    )
    url: EmptyObject = Field(
        default_factory=EmptyObject, description="url設定"
    )



from typing import Literal
from pydantic import Field

from ...models import RichTextItem
from ._base_property import BaseProperty


class TitleProperty(BaseProperty[Literal["title"]]):
    """Notionのtitleプロパティ"""

    type: Literal["title"] = Field("title", description="プロパティタイプ")

    title: list[RichTextItem] = Field(..., description="タイトルのRichText配列")

    def get_display_value(self) -> str | None:
        """タイトルの内容を取得

        Returns:
            str | None: タイトルの内容を連結した文字列。タイトルが空の場合はNone
        """
        if len(self.title) == 0:
            return None
        return "".join(item.plain_text for item in self.title)



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class LastVisitedTimePropertyConfig(
    BasePropertyConfig[Literal["last_visited_time"]]
):
    """Notionのlast_visited_timeプロパティ設定

    TypeScript: LastVisitedTimePropertyConfigurationRequest
    フィールドは空オブジェクトで構成されます。
    """

    type: Literal["last_visited_time"] = Field(
        "last_visited_time", description="プロパティタイプ"
    )
    last_visited_time: EmptyObject = Field(
        default_factory=EmptyObject, description="last_visited_time設定"
    )




from typing import Literal
from pydantic import Field, StrictStr

from ._base_property import BaseProperty


class UrlProperty(BaseProperty[Literal["url"]]):
    """Notionのurlプロパティ"""

    type: Literal["url"] = Field("url", description="プロパティタイプ")

    url: StrictStr | None = Field(None, description="URL（設定されていない場合はnull）")

    def get_display_value(self) -> str | None:
        """URLを取得

        Returns:
            StrictStr | None: URL（未設定の場合はnull）
        """
        return self.url



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class RichTextPropertyConfig(
    BasePropertyConfig[Literal["rich_text"]]
):
    """Notionのrich_textプロパティ設定"""

    type: Literal["rich_text"] = Field(
        "rich_text", description="プロパティタイプ"
    )
    rich_text: EmptyObject = Field(
        default_factory=EmptyObject, description="rich_text設定"
    )



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class FilesPropertyConfig(BasePropertyConfig[Literal["files"]]):
    """Notionのfilesプロパティ設定"""

    type: Literal["files"] = Field(
        "files", description="プロパティタイプ"
    )
    files: EmptyObject = Field(
        default_factory=EmptyObject, description="files設定"
    )



from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field, StrictStr

from .primitives import Annotations, Equation, Mention, Text


class RichTextItem(BaseModel):
    """NotionのRichText要素"""

    type: Literal["text", "mention", "equation"] = Field(
        ..., description="RichTextタイプ"
    )
    text: Text | None = Field(None, description="textタイプの内容")
    mention: Mention | None = Field(None, description="mentionタイプの内容")
    equation: Equation | None = Field(None, description="equationタイプの内容")
    annotations: Annotations = Field(..., description="アノテーション情報")
    plain_text: StrictStr = Field(..., description="プレーンテキスト")
    href: StrictStr | None = Field(None, description="リンクURL")

    def get_plain_text(self) -> str:
        """RichTextItemのプレーンテキストを取得"""
        return self.plain_text

    def to_markdown(self) -> str:
        """RichTextItemをMarkdown形式に変換"""
        text = self.plain_text

        # Apply code annotation exclusively (code blocks can't have other formatting)
        if self.annotations.code:
            # For code, we only need to escape backticks
            escaped_text = text.replace("`", "\\`")
            text = f"`{escaped_text}`"
            if self.href:
                text = f"[{text}]({self.href})"
            return text

        # Preserve leading and trailing spaces while applying formatting
        leading_space = ""
        trailing_space = ""

        # Extract leading spaces
        if text and text[0].isspace():
            leading_space = text[0]
            text = text[1:]

        # Extract trailing spaces
        if text and text[-1].isspace():
            trailing_space = text[-1]
            text = text[:-1]

        # Apply bold and italic (combined for better markdown)
        if text:  # Only apply formatting if there's non-space content
            if self.annotations.bold and self.annotations.italic:
                text = f"***{text}***"
            elif self.annotations.bold:
                text = f"**{text}**"
            elif self.annotations.italic:
                text = f"*{text}*"

            # Apply strikethrough
            if self.annotations.strikethrough:
                text = f"~~{text}~~"

        # Apply link
        if self.href and text:
            text = f"[{text}]({self.href})"

        # Restore leading and trailing spaces
        return leading_space + text + trailing_space


def rich_text_to_markdown(rich_text: list[RichTextItem]) -> str:
    """RichTextItemのリストをMarkdown文字列に変換"""
    return "".join([item.to_markdown() for item in rich_text])



from typing import Literal
from pydantic import Field, StrictBool

from ._base_property import BaseProperty


class CheckboxProperty(BaseProperty[Literal["checkbox"]]):
    """Notionのcheckboxプロパティ"""

    type: Literal["checkbox"] = Field("checkbox", description="プロパティタイプ")

    checkbox: StrictBool = Field(False, description="チェックボックスの状態")

    def get_display_value(self) -> bool:
        """チェックボックスの状態を取得

        Returns:
            StrictBool | None: チェックボックスの状態（True or False）
        """
        return self.checkbox



from typing import Literal

from pydantic import Field, StrictStr

from ._base_property import BaseProperty


class CreatedTimeProperty(BaseProperty[Literal["created_time"]]):
    """Notionのcreated_timeプロパティ"""

    type: Literal["created_time"] = Field(
        "created_time", description="プロパティタイプ"
    )

    created_time: StrictStr = Field(..., description="作成日時（ISO 8601形式）")

    def get_display_value(self) -> str:
        """作成日時を取得

        Returns:
            StrictStr | None: 作成日時（未設定の場合はnull）
        """
        return self.created_time



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class DatePropertyConfig(BasePropertyConfig[Literal["date"]]):
    """Notionのdateプロパティ設定"""

    type: Literal["date"] = Field(
        "date", description="プロパティタイプ"
    )
    date: EmptyObject = Field(
        default_factory=EmptyObject, description="date設定"
    )



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class CreatedTimePropertyConfig(
    BasePropertyConfig[Literal["created_time"]]
):
    """Notionのcreated_timeプロパティ設定"""

    type: Literal["created_time"] = Field(
        "created_time", description="プロパティタイプ"
    )
    created_time: EmptyObject = Field(
        default_factory=EmptyObject, description="created_time設定"
    )



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class CreatedByPropertyConfig(
    BasePropertyConfig[Literal["created_by"]]
):
    """Notionのcreated_byプロパティ設定"""

    type: Literal["created_by"] = Field(
        "created_by", description="プロパティタイプ"
    )
    created_by: EmptyObject = Field(
        default_factory=EmptyObject, description="created_by設定"
    )



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject


class LastEditedByPropertyConfig(
    BasePropertyConfig[Literal["last_edited_by"]]
):
    """Notionのlast_edited_byプロパティ設定"""

    type: Literal["last_edited_by"] = Field(
        "last_edited_by", description="プロパティタイプ"
    )
    last_edited_by: EmptyObject = Field(
        default_factory=EmptyObject, description="last_edited_by設定"
    )
